/*****************************************************************************
 * FairKeys.cs: FairKeys 0.3
 *****************************************************************************
 * Copyright (C) 2004 Jon Lech Johansen <jon-vl@nanocrew.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 *****************************************************************************/

using System;
using System.IO;
using System.Collections.Specialized;

class FairKeys
{
    private static void WriteKeys( NameValueCollection nvc,
                                   string strDsid, string strDir )
    {
        uint iDsid = Convert.ToUInt32( strDsid );

        Console.WriteLine( "Key directory: {0}", strDir );

        string [] KeyIDs = nvc.AllKeys;
        for( int i = 0; i < KeyIDs.Length; i++ )
        {
            BinaryWriter bwr;

            string strFile = String.Format( "{0:X8}.{1:D3}",
                iDsid, Convert.ToUInt32( KeyIDs[ i ] ) );
            string strPath = String.Format( "{0}{1}{2}",
                strDir, Path.DirectorySeparatorChar, strFile );

            Console.WriteLine( "Writing key: {0}", strFile );

            bwr = new BinaryWriter( File.OpenWrite( strPath ) );
            byte [] Key = FairStore.HexStringToBytes( nvc.Get( KeyIDs[ i ] ) );
            bwr.Write( Key );
            bwr.Close();
        }
    }

    // CHANGED(1): iOpener
	// public static void Main( string[] args )
	public static void GetKeys( string[] args )
    {
        if( args.Length < 2 || args.Length > 4 )
        {
            Console.WriteLine( "Usage: FairKeys <AppleID> <Password> [AccountKind] [GUID]" );
            return;
        }

        try
        {
            int AccountKind = 0;
            string strGUID = null;
            FairStore fs = new FairStore();

            if( args.Length > 2 )
            {
                for( int i = 2; i < args.Length; i++ )
                {
                    if( args[ i ].Length == 1 )
                    {
                        AccountKind = Convert.ToInt32( args[ i ] );
                    }
                    else
                    {
                        strGUID = args[ i ];
                    }
                }
            }

            bool bUnix = Environment.OSVersion.ToString().IndexOf( "Unix" ) != -1;
            string strHome = bUnix ? Environment.GetEnvironmentVariable( "HOME" ) :
                Environment.GetFolderPath( Environment.SpecialFolder.ApplicationData );
            string strDir = String.Format( "{0}{1}{2}drms",
                strHome, Path.DirectorySeparatorChar, bUnix ? "." : "" );

            if( !Directory.Exists( strDir ) )
                Directory.CreateDirectory( strDir );

            string strPath = String.Format( "{0}{1}GUID",
                strDir, Path.DirectorySeparatorChar );

            if( File.Exists( strPath ) )
            {
                StreamReader sr = new StreamReader( strPath );
                fs.GUID = sr.ReadToEnd();
                sr.Close();

                Console.WriteLine( "GUID exists, deauthorizing computer:" +
                    System.Environment.NewLine + fs.GUID );

                fs.Login( args[ 0 ], args[ 1 ], AccountKind );
                fs.DeauthorizeMachine();

                File.Delete( strPath );
            }
            else
            {
                if( strGUID == null )
                {
                    Random rnd = new Random();
                    strGUID = String.Format( "{0:X8}", rnd.Next() );
                    for( int i = 0; i < 6; i++ )
                        strGUID += String.Format( ".{0:X8}", rnd.Next() );
                }
                fs.GUID = strGUID;

                Console.WriteLine( "Authorizing computer, GUID:" +
                    System.Environment.NewLine + strGUID );

                fs.Login( args[ 0 ], args[ 1 ], AccountKind );
                WriteKeys( fs.AuthorizeMachine(), fs.Dsid, strDir );

                StreamWriter sw = new StreamWriter( strPath );
                sw.Write( strGUID );
                sw.Close();
            }
         }
         catch( Exception e )
         {
             Console.WriteLine( "Exception: " + e.Message );
         }
    }
}
